/*
 * Copyright (C) 2023 CERN (www.cern.ch)
 * Author: Adam Wujek
 *
 * Released according to the GNU GPL, version 2 or any later version.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <getopt.h>
#include <unistd.h>
#include <ppsi/ppsi.h>
#include <minipc.h>


#define PPSI_EXPORT_STRUCTURES
#include "ppsi_exports.h"

#define MINIPC_TIMEOUT 5000

static int verbose = 0;

static struct minipc_ch *ptp_ch;

enum input_arg {
	arg_help = 'Z' + 1, /* avoid conflicts with short options */
	arg_alpha,
	arg_prio1,
	arg_prio2,
	arg_tracking,
	arg_verbose,
};
	
static struct option long_opts[] = {
	{"help",	                no_argument,       NULL, arg_help},
	{"alpha",                       required_argument, NULL, arg_alpha},
	{"prio1",                       required_argument, NULL, arg_prio1},
	{"priority1",                   required_argument, NULL, arg_prio1},
	{"prio2",                       required_argument, NULL, arg_prio2},
	{"priority2",                   required_argument, NULL, arg_prio2},
	{"tracking",                    required_argument, NULL, arg_tracking},
	{"verbose",                     optional_argument, NULL, arg_verbose},
	{NULL, 0, NULL, 0}
};

void help(char *prgname)
{
	fprintf(stderr, "%s: Use: \"%s [-v] [-h] [option]\n",
		prgname, prgname);
	fprintf(stderr,
		"The program has the following options:\n"
		"  -h|--help         - print help\n"
		"  -v|--verbose      - verbose output\n"
		"Global parameters:\n"
		"  --alpha=<alpha64bit>\n"
		"                    - change the currently used alpha\n"
		"  --prio1=<num>\n"
		"  --priority1=<num> - set priority 1 to <num>\n"
		"  --prio2=<num>\n"
		"  --priority2=<num> - set priority 2 to <num>\n"
		"  --tracking=<on|off|enable|disable|1|0>\n"
		"                    - enable/disable tracking in servo\n"
		"\n\n"
		"Version: " __GIT_VER__ " compiled by " __GIT_USR__ " on " __DATE__ ", " __TIME__ "\n"
	);
	exit(1);
}


void ppsi_connect_minipc(void)
{
	if (ptp_ch) {
		/* close minipc, if connected before */
		minipc_close(ptp_ch);
	}
	ptp_ch = minipc_client_create("ptpd", 0);
	if (!ptp_ch) {
		fprintf(stderr, "Can't establish WRIPC connection to the PTP "
			 "daemon!\n");
		exit(1);
	}
}

void init_shm(void)
{
	ppsi_connect_minipc();
}


static void enable_disable_tracking(int onoff) {

	int rval;
	
	printf("%s: onoff %d\n", __func__, onoff);
	minipc_call(ptp_ch, MINIPC_TIMEOUT, &__rpcdef_cmd, &rval, 1, onoff);
}

static int ppsi_set_param(int param, int val) {

	int rval = 0;
	
	printf("%s: set param %d val %d\n", __func__, param, val);
	minipc_call(ptp_ch, MINIPC_TIMEOUT, &ppsiexp_update_param_cmd, &rval,
		    param, val);

	return rval;
}

static int ppsi_set_param64(int param, int64_t val64) {

	int rval = 0;
	
	printf("%s: set param %d val %lld\n", __func__, param, val64);
	minipc_call(ptp_ch, MINIPC_TIMEOUT, &ppsiexp_update_param64_cmd, &rval,
		    param, val64);

	return rval;
}

int main(int argc, char *argv[])
{
	int opt;
	int tmp;
	int ret;

	/* If no params print help */
	if (argc == 1)
		help(argv[0]);

	while ((opt = getopt_long(argc, argv, "vh", long_opts, NULL)) != -1) {
		if (opt == 'h' || opt == arg_help )
			help(argv[0]);
		if (opt == '?') {
			/* Unrecognized option, exit */
			exit(1);
		}

		if (opt == 'v') {
			verbose = 1;
			printf("Enable verbose mode\n");
		}

		if (opt == arg_verbose) {
			if (!strcmp(optarg, "off") || !strcmp(optarg, "0")) {
				verbose = 0;
				printf("Enable verbose mode\n");
			} else {
				verbose = 1;
				printf("Disable verbose mode\n");
			}
		}
	}

	init_shm();

	/* Reset position of argument for getopt_long */
	optind = 0;

	while ((opt = getopt_long(argc, argv, "vh", long_opts, NULL)) != -1) {

		switch (opt) {
		case arg_alpha:
			if (verbose)
				printf("Setting alpha to %lld\n",
				       atoll(optarg));
			ret = ppsi_set_param64(PPSIEXP_PARAM64_ALPHA_CMD,
					       atoll(optarg));
			if (ret) {
				fprintf(stderr, "Error setting alpha to %lld\n",
					atoll(optarg));
				exit (1);
			}
			break;

		case arg_prio1:
			tmp = atoi(optarg);
			if (verbose)
				printf("Setting priority1 to %d\n", tmp);
			ret = ppsi_set_param(PPSIEXP_PARAM_PRIORITY1_CMD, tmp);
			if (ret) {
				fprintf(stderr, "Error setting priority1 to "
					"%d\n", tmp);
				exit (1);
			}
			break;
		case arg_prio2:
			tmp = atoi(optarg);
			if (verbose)
				printf("Setting priority2 to %d\n", tmp);
			ret = ppsi_set_param(PPSIEXP_PARAM_PRIORITY2_CMD, tmp);
			if (ret) {
				fprintf(stderr, "Error setting priority2 to "
					"%d\n", tmp);
				exit (1);
			}
			break;
		case arg_tracking:
			{
				int tracking;
				if (!strcmp(optarg, "on")
				    || !strcmp(optarg, "enable")
				    || !strcmp(optarg, "1")
				   ) {
					tracking = 1;
				} else if (!strcmp(optarg, "off")
					   || !strcmp(optarg, "disable")
					   || !strcmp(optarg, "0")
					  ) {
					tracking = 0;
				} else {
					fprintf(stderr, "Unknown value (%s) for"
					        " tracking\n", optarg);
					exit (1);
				}
				if (verbose)
					printf("%s tracking\n",
					       tracking ? "Enable" : "Disable");

				enable_disable_tracking(tracking);
				break;
			}
		default:
			break;
		}
	}

	return 0;
}
